#if defined(__clang__)
#pragma clang diagnostic push
/* The next line is equivalent to #elif DCMTK_DIAGNOSTIC_MIN_GCC_VERSION(4,6,0) */
#elif ( defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6 ) ) )
#pragma GCC diagnostic push
#elif defined(_MSC_VER)
#pragma warning(push)
#endif
