#if defined(__clang__)
#pragma clang diagnostic ignored "-Wshadow"
/* The next line is equivalent to #elif DCMTK_DIAGNOSTIC_MIN_GCC_VERSION(4,6,0) */
#elif ( defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6 ) ) )
#pragma GCC diagnostic ignored "-Wshadow"
// Visual Studio doesn't seem to have such a warning yet, uncomment
// and replace ???? when it does
//#elif DCMTK_DIAGNOSTIC_MIN_MSC_VER(????)
//#pragma warning(disable: ????)
#endif
